@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-
+from django.http import HttpResponseRedirect
 from datetime import datetime
 import logging
 logger = logging.getLogger("sanza_crm")
@@ -16,3 +16,6 @@ def log_error(view_func):
 def now_rounded():
     now = datetime.now()
     return datetime(now.year, now.month, now.day, now.hour, now.minute, 0, 0)
+
+class HttpResponseRedirectMailtoAllowed(HttpResponseRedirect):
+    allowed_schemes = ['http', 'https', 'ftp', 'mailto']
\ No newline at end of file
